# -*- coding: utf-8 -*-
import hashlib
import logging
from pathlib import Path

from click import confirm

logger = logging.getLogger(__name__)


def sha1(file: Path) -> str:
    """Returns SHA1 hash of file"""
    return hashlib.sha1(file.read_bytes()).hexdigest()


def rename(file: Path, new_name: str):
    """Ranames the given file with the given new name"""
    logger.debug(f"Renaming file: {file.name} to {new_name}")

    while True:
        try:
            file.rename(f"{file.parent / new_name}")
            break
        except PermissionError:
            logger.error(
                f"A permission error has occurred for file: `{file}`. "
                "It is likely that the file you are trying to rename "
                "is currently opened by another user."
            )

            try_again = confirm(
                "File could not be renamed. Would you like to try again?", default=True
            )

            if not try_again:
                break



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pcuf` package."""
import hashlib
import warnings

import pytest

import pcuf
from tests.utils import TEST_DIR


@pytest.fixture
def file(request):
    test_file = TEST_DIR / "foo.txt"
    content = "hello"

    with open(str(test_file), "w+") as f:
        f.write(content)

    file_hash = hashlib.sha1(test_file.read_bytes()).hexdigest()

    def teardown():
        if test_file.is_file():
            test_file.unlink()

    request.addfinalizer(teardown)
    return dict(file=test_file, content=content, hash=file_hash)


def test_file_hash(file):
    """Test pcuf.file.sha1"""
    file_hash = file["hash"]
    cal_hash = pcuf.file.sha1(file["file"])

    assert file_hash == cal_hash


def test_file_rename(file):
    """Test pcuf.file.rename"""
    renamed_file = TEST_DIR / "foobar.txt"

    assert file["file"].is_file()
    assert not renamed_file.is_file()

    pcuf.file.rename(file["file"], "foobar.txt")
    assert renamed_file.is_file()

    renamed_file.unlink()
    assert not renamed_file.is_file()


def test_deprecation_warning():
    """Tests pcuf.warnings.deprecate """

    with warnings.catch_warnings(record=True) as warns:
        # Cause all warnings to always be triggered.
        warnings.simplefilter("always")

        # Trigger deprecation warning.
        pcuf.warnings.deprecate(
            "Deprecation Test.",
            version=pcuf.__version__,
            link_uid="fhQbw",
            link_file="gist-test",
        )

        assert len(warns) == 1
        assert issubclass(warns[-1].category, DeprecationWarning)
        assert "Deprecation Test." in str(warns[-1].message)
        assert pcuf.__version__ in str(warns[-1].message)


def test_retry_function():
    """Tests pcuf.functions.retry"""

    def completes_on_third_call():
        if completes_on_third_call.state < 2:
            completes_on_third_call.state += 1
            raise TypeError

        return 1

    completes_on_third_call.state = 0
    value = pcuf.functions.retry(completes_on_third_call)

    assert completes_on_third_call.state == 2
    assert value == 1

    def always_fail():
        always_fail.state += 1
        raise TypeError

    always_fail.state = 0

    with pytest.raises(TypeError):
        pcuf.functions.retry(always_fail, retries=6)

    assert always_fail.state == 6



# -*- coding: utf-8 -*-

from . import file, functions, time, warnings

__author__ = """Alberto J. Marin"""
__email__ = "alberto@ajmar.in"
__version__ = "0.1.1"

__all__ = ["file", "functions", "time", "warnings"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "xlrd>=1.2.0",
    "pandas>=0.24.1",
    "requests>=2.21.0",
    "click>=7.0",
    "openpyxl>=2.5.12",
]

setup_requirements = ["pytest-runner"]

test_requirements = ["pytest"]

setup(
    author="Alberto J. Marin",
    author_email="alberto@ajmar.in",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="A personal collection of useful and frequently used Python functions.",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="pcuf",
    name="pcuf",
    packages=find_packages(include=["pcuf"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/git-albertomarin/pcuf",
    version="0.1.1",
    zip_safe=False,
)



# -*- coding: utf-8 -*-
import warnings
from typing import Union

warnings.simplefilter("always", DeprecationWarning)


def deprecate(
    message: str,
    version: Union[int, float, str] = None,
    link_uid: str = None,
    link_file: str = None,
):
    """Displays a deprecation warning"""
    if version:
        message += "\nIt will be compatible before version {}.".format(version)
    if link_uid and link_file:
        message += "\nRead more <https://git.io/{}#file-{}-md>".format(
            link_uid, link_file
        )
    warnings.warn(DeprecationWarning(message), stacklevel=2)



# -*- coding: utf-8 -*-
import logging
from typing import Callable

logger = logging.getLogger(__name__)


def retry(func: Callable, retries: int = 3, *args, **kwargs):
    """Attempts to call a given function that may throw an exception x number of times,
     or until it runs without error."""
    err = None
    for idx in range(retries):
        try:
            logger.debug(f"Executing {func.__name__} function...{idx + 1}")
            return func(*args, **kwargs)
        except Exception as e:
            logger.debug(
                f"{func.__name__} function threw {e.__class__.__name__}. Retrying..."
            )
            err = e

    logger.debug(f"{func.__name__}  execution failed! Raising exception...")
    raise err
